@@ -3,7 +3,7 @@
  * @brief BSON Declarations
  */
 
-/*    Copyright 2009-2011 10gen Inc.
+/*    Copyright 2009-2012 10gen Inc.
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -21,13 +21,60 @@
 #ifndef BSON_H_
 #define BSON_H_
 
-#include "platform.h"
 #include <time.h>
-#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <stdarg.h>
 
+#ifdef __GNUC__
+#define MONGO_INLINE static __inline__
+#define MONGO_EXPORT
+#else
+#define MONGO_INLINE static
+#ifdef MONGO_STATIC_BUILD
+#define MONGO_EXPORT
+#elif defined(MONGO_DLL_BUILD)
+#define MONGO_EXPORT __declspec(dllexport)
+#else
+#define MONGO_EXPORT __declspec(dllimport)
+#endif
+#endif
+
+#ifdef __cplusplus
+#define MONGO_EXTERN_C_START extern "C" {
+#define MONGO_EXTERN_C_END }
+#else
+#define MONGO_EXTERN_C_START
+#define MONGO_EXTERN_C_END
+#endif
+
+#if defined(MONGO_HAVE_STDINT) || __STDC_VERSION__ >= 199901L
+#include <stdint.h>
+#elif defined(MONGO_HAVE_UNISTD)
+#include <unistd.h>
+#elif defined(MONGO_USE__INT64)
+typedef __int64 int64_t;
+typedef unsigned __int64 uint64_t;
+#elif defined(MONGO_USE_LONG_LONG_INT)
+typedef long long int int64_t;
+typedef unsigned long long int uint64_t;
+#else
+#error Must compile with c99 or define MONGO_HAVE_STDINT, MONGO_HAVE_UNISTD, MONGO_USE__INT64, or MONGO_USE_LONG_LONG_INT.
+#endif
+
+#ifdef MONGO_BIG_ENDIAN
+#define bson_little_endian64(out, in) ( bson_swap_endian64(out, in) )
+#define bson_little_endian32(out, in) ( bson_swap_endian32(out, in) )
+#define bson_big_endian64(out, in) ( memcpy(out, in, 8) )
+#define bson_big_endian32(out, in) ( memcpy(out, in, 4) )
+#else
+#define bson_little_endian64(out, in) ( memcpy(out, in, 8) )
+#define bson_little_endian32(out, in) ( memcpy(out, in, 4) )
+#define bson_big_endian64(out, in) ( bson_swap_endian64(out, in) )
+#define bson_big_endian32(out, in) ( bson_swap_endian32(out, in) )
+#endif
+
 MONGO_EXTERN_C_START
 
 #define BSON_OK 0
@@ -84,12 +131,12 @@ typedef struct {
 } bson_iterator;
 
 typedef struct {
-    char *data;
-    char *cur;
-    int dataSize;
-    bson_bool_t finished;
-    int stack[32];
-    int stackPos;
+    char *data;    /**< Pointer to a block of data in this BSON object. */
+    char *cur;     /**< Pointer to the current position. */
+    int dataSize;  /**< The number of bytes allocated to char *data. */
+    bson_bool_t finished; /**< When finished, the BSON object can no longer be modified. */
+    int stack[32];        /**< A stack used to keep track of nested BSON elements. */
+    int stackPos;         /**< Index of current stack position. */
     int err; /**< Bitfield representing errors or warnings on this buffer */
     char *errstr; /**< A string representation of the most recent error or warning. */
 } bson;
@@ -112,8 +159,9 @@ typedef struct {
    READING
    ------------------------------ */
 
-MONGO_EXPORT bson* bson_create();
+MONGO_EXPORT bson* bson_create( void );
 MONGO_EXPORT void  bson_dispose(bson* b);
+
 /**
  * Size of a BSON object.
  *
@@ -129,14 +177,14 @@ MONGO_EXPORT int bson_buffer_size( const bson *b );
  *
  * @param b the BSON object to print.
  */
-void bson_print( bson *b );
+MONGO_EXPORT void bson_print( const bson *b );
 
 /**
  * Return a pointer to the raw buffer stored by this bson object.
  *
  * @param b a BSON object
  */
-const char *bson_data( bson *b );
+MONGO_EXPORT const char *bson_data( const bson *b );
 
 /**
  * Print a string representation of a BSON object.
@@ -144,7 +192,7 @@ const char *bson_data( bson *b );
  * @param bson the raw data to print.
  * @param depth the depth to recurse the object.x
  */
-void bson_print_raw( const char *bson , int depth );
+MONGO_EXPORT void bson_print_raw( const char *bson , int depth );
 
 /**
  * Advance a bson_iterator to the named field.
@@ -158,7 +206,7 @@ void bson_print_raw( const char *bson , int depth );
 MONGO_EXPORT bson_type bson_find( bson_iterator *it, const bson *obj, const char *name );
 
 
-MONGO_EXPORT bson_iterator* bson_iterator_create();
+MONGO_EXPORT bson_iterator* bson_iterator_create( void );
 MONGO_EXPORT void bson_iterator_dispose(bson_iterator*);
 /**
  * Initialize a bson_iterator.
@@ -175,7 +223,7 @@ MONGO_EXPORT void bson_iterator_init( bson_iterator *i , const bson *b );
  * @param i the bson_iterator to initialize.
  * @param buffer the buffer to point to.
  */
-void bson_iterator_from_buffer( bson_iterator *i, const char *buffer );
+MONGO_EXPORT void bson_iterator_from_buffer( bson_iterator *i, const char *buffer );
 
 /* more returns true for eoo. best to loop with bson_iterator_next(&it) */
 /**
@@ -185,7 +233,7 @@ void bson_iterator_from_buffer( bson_iterator *i, const char *buffer );
  *
  * @return  returns true if there is more data.
  */
-bson_bool_t bson_iterator_more( const bson_iterator *i );
+MONGO_EXPORT bson_bool_t bson_iterator_more( const bson_iterator *i );
 
 /**
  * Point the iterator at the next BSON object.
@@ -221,7 +269,7 @@ MONGO_EXPORT const char *bson_iterator_key( const bson_iterator *i );
  *
  * @return  the value of the current BSON object.
  */
-const char *bson_iterator_value( const bson_iterator *i );
+MONGO_EXPORT const char *bson_iterator_value( const bson_iterator *i );
 
 /* these convert to the right type (return 0 if non-numeric) */
 /**
@@ -261,7 +309,7 @@ MONGO_EXPORT int64_t bson_iterator_long( const bson_iterator *i );
  *
  * @return the value of the current BSON object.
  */
-bson_timestamp_t bson_iterator_timestamp( const bson_iterator *i );
+MONGO_EXPORT bson_timestamp_t bson_iterator_timestamp( const bson_iterator *i );
 MONGO_EXPORT int bson_iterator_timestamp_time( const bson_iterator *i );
 MONGO_EXPORT int bson_iterator_timestamp_increment( const bson_iterator *i );
 
@@ -390,7 +438,7 @@ MONGO_EXPORT bson_date_t bson_iterator_date( const bson_iterator *i );
  *
  * @return the time value of the current BSON object.
  */
-time_t bson_iterator_time_t( const bson_iterator *i );
+MONGO_EXPORT time_t bson_iterator_time_t( const bson_iterator *i );
 
 /**
  * Get the length of the BSON binary object currently pointed to by the
@@ -450,7 +498,7 @@ MONGO_EXPORT const char *bson_iterator_regex_opts( const bson_iterator *i );
  * @param i the bson_iterator.
  * @param sub the BSON subobject destination.
  */
-void bson_iterator_subobject( const bson_iterator *i, bson *sub );
+MONGO_EXPORT void bson_iterator_subobject( const bson_iterator *i, bson *sub );
 
 /**
  * Get a bson_iterator that on the BSON subobject.
@@ -490,7 +538,7 @@ MONGO_EXPORT void bson_oid_gen( bson_oid_t *oid );
  *
  * @param func a pointer to a function that returns an int.
  */
-void bson_set_oid_fuzz( int ( *func )( void ) );
+MONGO_EXPORT void bson_set_oid_fuzz( int ( *func )( void ) );
 
 /**
  * Set a function to be used to generate the incrementing part
@@ -499,14 +547,14 @@ void bson_set_oid_fuzz( int ( *func )( void ) );
  *
  * @param func a pointer to a function that returns an int.
  */
-void bson_set_oid_inc( int ( *func )( void ) );
+MONGO_EXPORT void bson_set_oid_inc( int ( *func )( void ) );
 
 /**
  * Get the time a bson_oid_t was created.
  *
  * @param oid the bson_oid_t.
  */
-time_t bson_oid_generated_time( bson_oid_t *oid ); /* Gives the time the OID was created */
+MONGO_EXPORT time_t bson_oid_generated_time( bson_oid_t *oid ); /* Gives the time the OID was created */
 
 /* ----------------------------
    BUILDING
@@ -582,7 +630,7 @@ MONGO_EXPORT void bson_destroy( bson *b );
  * @return the empty initialized BSON object.
  */
 /* returns pointer to static empty bson object */
-bson *bson_empty( bson *obj );
+MONGO_EXPORT bson *bson_empty( bson *obj );
 
 /**
  * Make a complete copy of the a BSON object.
@@ -613,7 +661,7 @@ MONGO_EXPORT int bson_append_oid( bson *b, const char *name, const bson_oid_t *o
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_new_oid( bson *b, const char *name );
+MONGO_EXPORT int bson_append_new_oid( bson *b, const char *name );
 
 /**
  * Append an int to a bson.
@@ -669,7 +717,7 @@ MONGO_EXPORT int bson_append_string( bson *b, const char *name, const char *str
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_string_n( bson *b, const char *name, const char *str, int len );
+MONGO_EXPORT int bson_append_string_n( bson *b, const char *name, const char *str, int len );
 
 /**
  * Append a symbol to a bson.
@@ -692,7 +740,7 @@ MONGO_EXPORT int bson_append_symbol( bson *b, const char *name, const char *str
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_symbol_n( bson *b, const char *name, const char *str, int len );
+MONGO_EXPORT int bson_append_symbol_n( bson *b, const char *name, const char *str, int len );
 
 /**
  * Append code to a bson.
@@ -716,7 +764,7 @@ MONGO_EXPORT int bson_append_code( bson *b, const char *name, const char *str );
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_code_n( bson *b, const char *name, const char *str, int len );
+MONGO_EXPORT int bson_append_code_n( bson *b, const char *name, const char *str, int len );
 
 /**
  * Append code to a bson with scope.
@@ -741,7 +789,7 @@ MONGO_EXPORT int bson_append_code_w_scope( bson *b, const char *name, const char
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_code_w_scope_n( bson *b, const char *name, const char *code, int size, const bson *scope );
+MONGO_EXPORT int bson_append_code_w_scope_n( bson *b, const char *name, const char *code, int size, const bson *scope );
 
 /**
  * Append binary data to a bson.
@@ -819,7 +867,7 @@ MONGO_EXPORT int bson_append_bson( bson *b, const char *name, const bson *bson )
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_element( bson *b, const char *name_or_null, const bson_iterator *elem );
+MONGO_EXPORT int bson_append_element( bson *b, const char *name_or_null, const bson_iterator *elem );
 
 /**
  * Append a bson_timestamp_t value to a bson.
@@ -830,7 +878,7 @@ int bson_append_element( bson *b, const char *name_or_null, const bson_iterator
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_timestamp( bson *b, const char *name, bson_timestamp_t *ts );
+MONGO_EXPORT int bson_append_timestamp( bson *b, const char *name, bson_timestamp_t *ts );
 MONGO_EXPORT int bson_append_timestamp2( bson *b, const char *name, int time, int increment );
 
 /* these both append a bson_date */
@@ -854,7 +902,7 @@ MONGO_EXPORT int bson_append_date( bson *b, const char *name, bson_date_t millis
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_time_t( bson *b, const char *name, time_t secs );
+MONGO_EXPORT int bson_append_time_t( bson *b, const char *name, time_t secs );
 
 /**
  * Start appending a new object to a bson.
@@ -893,7 +941,7 @@ MONGO_EXPORT int bson_append_finish_object( bson *b );
  *
  * @return BSON_OK or BSON_ERROR.
  */
-int bson_append_finish_array( bson *b );
+MONGO_EXPORT int bson_append_finish_array( bson *b );
 
 void bson_numstr( char *str, int i );
 
@@ -911,14 +959,15 @@ typedef int (*bson_sprintf_func)( char *, const char *, ... );
 
 extern void *( *bson_malloc_func )( size_t );
 extern void *( *bson_realloc_func )( void *, size_t );
-extern void ( *bson_free )( void * );
+extern void ( *bson_free_func )( void * );
 
 extern bson_printf_func bson_printf;
 extern bson_fprintf_func bson_fprintf;
 extern bson_sprintf_func bson_sprintf;
-
 extern bson_printf_func bson_errprintf;
 
+MONGO_EXPORT void bson_free( void *ptr );
+
 /**
  * Allocates memory and checks return value, exiting fatally if malloc() fails.
  *
@@ -928,7 +977,7 @@ extern bson_printf_func bson_errprintf;
  *
  * @sa malloc(3)
  */
-void *bson_malloc( int size );
+MONGO_EXPORT void *bson_malloc( int size );
 
 /**
  * Changes the size of allocated memory and checks return value,
@@ -950,7 +999,7 @@ void *bson_realloc( void *ptr, int size );
  *
  * @return the old error handling function, or NULL.
  */
-bson_err_handler set_bson_err_handler( bson_err_handler func );
+MONGO_EXPORT bson_err_handler set_bson_err_handler( bson_err_handler func );
 
 /* does nothing if ok != 0 */
 /**
@@ -982,5 +1031,8 @@ void bson_builder_error( bson *b );
  */
 MONGO_EXPORT double bson_int64_to_double( int64_t i64 );
 
+MONGO_EXPORT void bson_swap_endian32( void *outp, const void *inp );
+MONGO_EXPORT void bson_swap_endian64( void *outp, const void *inp );
+
 MONGO_EXTERN_C_END
 #endif
